/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.sql.tx;

import java.sql.Connection;
import java.sql.SQLException;

import javax.sql.DataSource;

import woolpack.fn.Fn;

public class RollbackFn<C, R> implements Fn<C, R, SQLException> {
	private DataSource dataSource;
	
	public RollbackFn(final DataSource dataSource) {
		this.dataSource = dataSource;
	}

	public R exec(final C c) throws SQLException {
		SQLException e0 = null;
		try {
			final Connection connection = dataSource.getConnection();
			try {
				connection.rollback();
			} catch (final SQLException e1) {
				e0 = e1;
			} finally {
				connection.close();
			}
		} catch (final SQLException e1) {
			if (e0 == null) {
				e0 = e1;
			}
		}
		if (e0 != null) {
			throw e0;
		} else {
			return null;
		}
	}

	public DataSource getDataSource() {
		return dataSource;
	}
	public void setDataSource(final DataSource dataSource) {
		this.dataSource = dataSource;
	}
}
